// vi: set ts=4 sw=4 :

// vim: set tw=75 :



// engine_api.cpp - implementation of Half-Life engine functions



/*

 * Copyright (c) 2001-2003 Will Day <willday@hpgx.net>

 *

 *    This file is part of Metamod.

 *

 *    Metamod is free software; you can redistribute it and/or modify it

 *    under the terms of the GNU General Public License as published by the

 *    Free Software Foundation; either version 2 of the License, or (at

 *    your option) any later version.

 *

 *    Metamod is distributed in the hope that it will be useful, but

 *    WITHOUT ANY WARRANTY; without even the implied warranty of

 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU

 *    General Public License for more details.

 *

 *    You should have received a copy of the GNU General Public License

 *    along with Metamod; if not, write to the Free Software Foundation,

 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *

 *    In addition, as a special exception, the author gives permission to

 *    link the code of this program with the Half-Life Game Engine ("HL

 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,

 *    L.L.C ("Valve").  You must obey the GNU General Public License in all

 *    respects for all of the code used other than the HL Engine and MODs

 *    from Valve.  If you modify this file, you may extend this exception

 *    to your version of the file, but you are not obligated to do so.  If

 *    you do not wish to do so, delete this exception statement from your

 *    version.

 *

 */



#include <stdio.h>

#include <stdarg.h>



#include <extdll.h>



#include "engine_api.h"

#include "meta_api.h"

#include "log_plugin.h"



enginefuncs_t meta_engfuncs;



C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, 

		int *interfaceVersion) 

{

	LOG_DEVELOPER(PLID, "called: GetEngineFunctions; version=%d", 

			*interfaceVersion);

	if(!pengfuncsFromEngine) {

		LOG_ERROR(PLID, 

				"GetEngineFunctions called with null pengfuncsFromEngine");

		return(FALSE);

	}

	else if(*interfaceVersion != ENGINE_INTERFACE_VERSION) {

		LOG_ERROR(PLID, 

				"GetEngineFunctions version mismatch; requested=%d ours=%d", 

				*interfaceVersion, ENGINE_INTERFACE_VERSION);

		// Tell metamod what version we had, so it can figure out who is 

		// out of date.

		*interfaceVersion = ENGINE_INTERFACE_VERSION;

		return(FALSE);

	}

	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));

	return TRUE;

}

